@@ -28,6 +28,8 @@
 #include <klocale.h>
 
 #include "core/mixer.h"
+#include "core/ControlManager.h"
+#include "core/GlobalConfig.h"
 
 #include <pulse/glib-mainloop.h>
 #include <pulse/ext-stream-restore.h>
@@ -214,7 +216,7 @@ static void sink_cb(pa_context *c, const pa_sink_info *i, int eol, void *) {
 
     bool is_new = !outputDevices.contains(s.index);
     outputDevices[s.index] = s;
-    kDebug(67100) << "Got some info about sink: " << s.description;
+//     kDebug(67100) << "Got some info about sink: " << s.description;
 
     if (s_mixers.contains(KMIXPA_PLAYBACK)) {
         if (is_new)
@@ -267,7 +269,7 @@ static void source_cb(pa_context *c, const pa_source_info *i, int eol, void *) {
 
     bool is_new = !captureDevices.contains(s.index);
     captureDevices[s.index] = s;
-    kDebug(67100) << "Got some info about source: " << s.description;
+//     kDebug(67100) << "Got some info about source: " << s.description;
 
     if (s_mixers.contains(KMIXPA_CAPTURE)) {
         if (is_new)
@@ -298,7 +300,7 @@ static void client_cb(pa_context *c, const pa_client_info *i, int eol, void *) {
     }
 
     clients[i->index] = QString::fromUtf8(i->name);
-    kDebug(67100) << "Got some info about client: " << clients[i->index];
+    //kDebug(67100) << "Got some info about client: " << clients[i->index];
 }
 
 static void sink_input_cb(pa_context *c, const pa_sink_input_info *i, int eol, void *) {
@@ -347,7 +349,7 @@ static void sink_input_cb(pa_context *c, const pa_sink_input_info *i, int eol, v
 
     bool is_new = !outputStreams.contains(s.index);
     outputStreams[s.index] = s;
-    kDebug(67100) << "Got some info about sink input (playback stream): " << s.description;
+//     kDebug(67100) << "Got some info about sink input (playback stream): " << s.description;
 
     if (s_mixers.contains(KMIXPA_APP_PLAYBACK)) {
         if (is_new)
@@ -411,7 +413,7 @@ static void source_output_cb(pa_context *c, const pa_source_output_info *i, int
 
     bool is_new = !captureStreams.contains(s.index);
     captureStreams[s.index] = s;
-    kDebug(67100) << "Got some info about source output (capture stream): " << s.description;
+//     kDebug(67100) << "Got some info about source output (capture stream): " << s.description;
 
     if (s_mixers.contains(KMIXPA_APP_CAPTURE)) {
         if (is_new)
@@ -490,7 +492,7 @@ void ext_stream_restore_read_cb(pa_context *c, const pa_ext_stream_restore_info
 
 
     QString name = QString::fromUtf8(i->name);
-    kDebug(67100) << QString("Got some info about restore rule: '%1' (Device: %2)").arg(name).arg(i->device ? i->device : "None");
+//     kDebug(67100) << QString("Got some info about restore rule: '%1' (Device: %2)").arg(name).arg(i->device ? i->device : "None");
     restoreRule rule;
     rule.channel_map = i->channel_map;
     rule.volume = i->volume;
@@ -761,14 +763,7 @@ static devmap* get_widget_map(int type, int index)
 
 void Mixer_PULSE::emitControlsReconfigured()
 {
-    // Do not emit directly to ensure all connected slots are executed
-    // in their own event loop.
-	kDebug() << "PULSE emitControlsReconfigured: mixerId=" << _mixer->id();
-//	emit controlsReconfigured(_mixer->id());
-    QMetaObject::invokeMethod(this,
-                              "controlsReconfigured",
-                              Qt::QueuedConnection,
-                              Q_ARG(QString, _mixer->id()));
+    ControlManager::instance().announce(_mixer->id(), ControlChangeType::ControlList, getDriverName());
 }
 
 void Mixer_PULSE::addWidget(int index)
@@ -820,14 +815,6 @@ void Mixer_PULSE::removeAllWidgets()
         outputRoles.clear();
 
     m_mixDevices.clear();
-
-    // TODO cesken LET-THIS-CHECK  Delete should not be required after migration to shared_ptr
-//    MixSet::iterator iter;
-//    for (iter = m_mixDevices.begin(); iter != m_mixDevices.end(); ++iter)
-//    {
-//        delete *iter;
-//        m_mixDevices.erase(iter);
-//    }
     emitControlsReconfigured();
 }
 
@@ -835,19 +822,25 @@ void Mixer_PULSE::addDevice(devinfo& dev, bool isAppStream)
 {
     if (dev.chanMask != Volume::MNONE) {
         MixSet *ms = 0;
+	bool isCapture = (m_devnum == KMIXPA_CAPTURE || m_devnum == KMIXPA_APP_CAPTURE);
         if (m_devnum == KMIXPA_APP_PLAYBACK && s_mixers.contains(KMIXPA_PLAYBACK))
             ms = s_mixers[KMIXPA_PLAYBACK]->getMixSet();
         else if (m_devnum == KMIXPA_APP_CAPTURE && s_mixers.contains(KMIXPA_CAPTURE))
             ms = s_mixers[KMIXPA_CAPTURE]->getMixSet();
 
-        Volume v(PA_VOLUME_NORM, PA_VOLUME_MUTED, true, false);
+        int maxVol = GlobalConfig::instance().volumeOverdrive ? PA_VOLUME_UI_MAX : PA_VOLUME_NORM;
+        Volume v(maxVol, PA_VOLUME_MUTED, true, false);
         v.addVolumeChannels(dev.chanMask);
         setVolumeFromPulse(v, dev);
         MixDevice* md = new MixDevice( _mixer, dev.name, dev.description, dev.icon_name, ms);
         if (isAppStream)
             md->setApplicationStream(true);
 
-        md->addPlaybackVolume(v);
+	kDebug() << "Adding Pulse volume " << dev.name << ", isCapture= " << isCapture << ", devnum=" << m_devnum;
+	if ( isCapture )
+	  md->addCaptureVolume(v);
+	else
+	  md->addPlaybackVolume(v);
         md->setMuted(dev.mute);
         m_mixDevices.append(md->addToPool());
     }
@@ -868,7 +861,7 @@ bool Mixer_PULSE::connectToDaemon()
     pa_mainloop_api *api = pa_glib_mainloop_get_api(s_mainloop);
     Q_ASSERT(api);
 
-    s_context = pa_context_new(api, "KMix KDE 4");
+    s_context = pa_context_new(api, "KMix");
     Q_ASSERT(s_context);
 
     if (pa_context_connect(s_context, NULL, PA_CONTEXT_NOFAIL, 0) < 0) {
@@ -1010,18 +1003,21 @@ int Mixer_PULSE::open()
         devmap::iterator iter;
         if (KMIXPA_PLAYBACK == m_devnum)
         {
+        	_id = "Playback Devices";
             m_mixerName = i18n("Playback Devices");
             for (iter = outputDevices.begin(); iter != outputDevices.end(); ++iter)
                 addDevice(*iter);
         }
         else if (KMIXPA_CAPTURE == m_devnum)
         {
+        	_id = "Capture Devices";
             m_mixerName = i18n("Capture Devices");
             for (iter = captureDevices.begin(); iter != captureDevices.end(); ++iter)
                 addDevice(*iter);
         }
         else if (KMIXPA_APP_PLAYBACK == m_devnum)
         {
+        	_id = "Playback Streams";
             m_mixerName = i18n("Playback Streams");
             for (iter = outputRoles.begin(); iter != outputRoles.end(); ++iter)
                 addDevice(*iter, true);
@@ -1030,6 +1026,7 @@ int Mixer_PULSE::open()
         }
         else if (KMIXPA_APP_CAPTURE == m_devnum)
         {
+        	_id = "Capture Streams";
             m_mixerName = i18n("Capture Streams");
             for (iter = captureStreams.begin(); iter != captureStreams.end(); ++iter)
                 addDevice(*iter);
