@@ -29,6 +29,7 @@
 
 #include "backends/mixer_backend.h"
 #include "backends/kmix-backends.cpp"
+#include "core/ControlManager.h"
 #include "core/volume.h"
 
 /**
@@ -63,6 +64,31 @@ QList<Mixer *>& Mixer::mixers()
     return s_mixers;
 }
 
+/**
+ * Returns whether there is at least one dynamic mixer active.
+ * @returns true, if at least one dynamic mixer is active
+ */
+bool Mixer::dynamicBackendsPresent()
+{
+  foreach ( Mixer* mixer, Mixer::mixers() )
+  {
+    if ( mixer->isDynamic() )
+      return true;
+  }
+  return false;
+}
+
+bool Mixer::pulseaudioPresent()
+{
+  foreach ( Mixer* mixer, Mixer::mixers() )
+  {
+    if ( mixer->getDriverName() == "PulseAudio" )
+      return true;
+  }
+  return false;
+}
+
+
 Mixer::Mixer( QString& ref_driverName, int device )
     : m_balance(0), _mixerBackend(0L), m_dynamic(false)
 {
@@ -111,16 +137,18 @@ Mixer* Mixer::findMixer( const QString& mixer_id)
 }
 
 
-/**
- * Set the card instance. Usually this will be 1, but if there is
- * more than one card with the same name install, then you need
- * to use 2, 3, ...
- */
-void Mixer::setCardInstance(int cardInstance)
-{
-    _cardInstance = cardInstance;
-    recreateId();
-}
+///**
+// * Set the card instance. Usually this will be 1, but if there is
+// * more than one card with the same name install, then you need
+// * to use 2, 3, ...
+// */
+//void Mixer::setCardInstance(int cardInstance)
+//{
+//    _cardInstance = cardInstance;
+//    recreateId();
+//    // DBusMixerWrapper must be called after recreateId(), as it uses the id
+//    new DBusMixerWrapper(this, dbusPath());
+//}
 
 void Mixer::recreateId()
 {
@@ -130,7 +158,7 @@ void Mixer::recreateId()
      * %2, the mixer name, is typically coming from an OS driver. It could contain colons.
      * %3, the mixer number, is a number: it does not contain colons.
      */
-    QString mixerName = getBaseName();
+    QString mixerName = _mixerBackend->getId();
     mixerName.replace(':','_');
     QString primaryKeyOfMixer = QString("%1::%2:%3")
             .arg(getDriverName())
@@ -142,10 +170,19 @@ void Mixer::recreateId()
     primaryKeyOfMixer.replace(' ','_');
     primaryKeyOfMixer.replace('=','_');
     _id = primaryKeyOfMixer;
+	kDebug() << "Early _id=" << _id;
 }
 
-const QString Mixer::dbusPath() {
-    return "/Mixers/" + QString::number( _mixerBackend->m_devnum );
+const QString Mixer::dbusPath()
+{
+	// This needs to be fixed from the very beginning, as MixDevice::dbusPath() also uses it. So once the first
+	// MixDevice is created,
+
+    //return QString("/Mixers/" + QString::number( _mixerBackend->m_devnum ) );
+    //return QString("/Mixers/" +  getDriverName() + "." + _mixerBackend->getId()).replace(" ", "x").replace(".", "_");
+	kDebug() << "Late _id=" << _id;
+	kDebug() << "handMade=" << QString("/Mixers/" +  getDriverName() + "." + _mixerBackend->getId()).replace(" ", "x").replace(".", "_");
+    return QString("/Mixers/" + _id.replace(" ", "x").replace(".", "_").replace(":", "_").replace("-", "_") );
 }
 
 void Mixer::volumeSave( KConfig *config )
@@ -182,7 +219,8 @@ void Mixer::volumeLoad( KConfig *config )
 		   continue;
 
        _mixerBackend->writeVolumeToHW( md->id(), md );
-       if ( md->isEnum() ) _mixerBackend->setEnumIdHW( md->id(), md->enumId() );
+       if ( md->isEnum() )
+    	   _mixerBackend->setEnumIdHW( md->id(), md->enumId() );
    }
 }
 
@@ -191,13 +229,25 @@ void Mixer::volumeLoad( KConfig *config )
  * Opens the mixer.
  * Also, starts the polling timer, for polling the Volumes from the Mixer.
  *
- * @return 0, if OK. An Mixer::ERR_ error code otherwise
+ * @param cardId The cardId Usually this will be 1, but if there is
+ * more than one card with the same name install, then you need
+ * to use 2, 3, ...
+ *
+ * @return true, if Mixer could be opened.
  */
-bool Mixer::openIfValid() {
+bool Mixer::openIfValid(int cardId)
+{
+	if (_mixerBackend == 0 )
+	{
+		// if we did not instantiate a suitable Backend, then Mixer is invalid
+		return false;
+	}
+
+	_cardInstance = cardId;
     bool ok = _mixerBackend->openIfValid();
     if ( ok )
     {
-        recreateId(); // TODO NOW : We actually cannot postpone it to here, due to ControlPool. Move to Mixer !!!. Actually recreateId() is supposed to be called later again, via setCardInstance()
+        recreateId();
         shared_ptr<MixDevice> recommendedMaster = _mixerBackend->recommendedMaster();
         if ( recommendedMaster.get() != 0 )
         {
@@ -213,8 +263,6 @@ bool Mixer::openIfValid() {
             setLocalMasterMD(noMaster); // no master
         }
         connect( _mixerBackend, SIGNAL(controlChanged()), SIGNAL(controlChanged()) );
-        connect( _mixerBackend, SIGNAL(controlsReconfigured(QString)), SIGNAL(controlsReconfigured(QString)) );
-    
         new DBusMixerWrapper(this, dbusPath());
     }
 
@@ -223,13 +271,11 @@ bool Mixer::openIfValid() {
 
 /**
  * Closes the mixer.
- * Also, stops the polling timer.
- *
- * @return 0 (always)
  */
-int Mixer::close()
+void Mixer::close()
 {
-  return _mixerBackend->close();
+	if ( _mixerBackend != 0)
+		_mixerBackend->close();
 }
 
 
@@ -333,22 +379,23 @@ void Mixer::setBalanceInternal(Volume& vol)
    }
 }
 
-// should return a name suitable for a human user to read (on a label, ...)
+/**
+ * Returns a name suitable for a human user to read (on a label, ...)
+ */
 QString Mixer::readableName()
 {
-  if ( _mixerBackend->m_mixerName.endsWith(":0")) {
-      QString finalName = _mixerBackend->m_mixerName.left(_mixerBackend->m_mixerName.length() - 2);
-      finalName = finalName.append(' ').arg(getCardInstance());
-      return finalName;
-  }
-  else
-     return _mixerBackend->m_mixerName;
+	QString finalName = _mixerBackend->getName();
+//	QString finalName = mixerName.left(mixerName.length() - 2);
+	if ( getCardInstance() > 1)
+		finalName = finalName.append(" %1").arg(getCardInstance());
+
+	return finalName;
 }
 
 
 QString Mixer::getBaseName()
 {
-  return _mixerBackend->m_mixerName;
+  return _mixerBackend->getName();
 }
 
 /**
@@ -461,7 +508,8 @@ shared_ptr<MixDevice> Mixer::getGlobalMasterMD(bool fallbackAllowed)
 	{
 		if ( md.get() == 0 )
 			continue; // invalid
-			firstDevice=md;
+
+		firstDevice=md;
 		if ( md->id() == _globalMasterCurrent.getControl() )
 		{
 			mdRet = md;
@@ -535,9 +583,13 @@ shared_ptr<MixDevice> Mixer::getMixdeviceById( const QString& mixdeviceID )
 void Mixer::commitVolumeChange( shared_ptr<MixDevice> md )
 {
   _mixerBackend->writeVolumeToHW(md->id(), md );
-   if (md->isEnum()) _mixerBackend->setEnumIdHW(md->id(), md->enumId() );
-   if ( md->captureVolume().hasSwitch() ) {
-      // Make sure to re-read the hardware, because seting capture might have failed.
+   if (md->isEnum())
+   {
+     _mixerBackend->setEnumIdHW(md->id(), md->enumId() );
+   }
+   if ( md->captureVolume().hasSwitch() )
+   {
+      // Make sure to re-read the hardware, because setting capture might have failed.
       // This is due to exclusive capture groups.
       // If we wouldn't do this, KMix might show a Capture Switch disabled, but
       // in reality the capture switch is still on.
@@ -545,8 +597,12 @@ void Mixer::commitVolumeChange( shared_ptr<MixDevice> md )
       // We also cannot rely on a notification from the driver (SocketNotifier), because
       // nothing has changed, and so there s nothing to notify.
       _mixerBackend->readSetFromHWforceUpdate();
+      kDebug() << "commiting a control with capture volume, that might announce: " << md->id();
       _mixerBackend->readSetFromHW();
    }
+      kDebug() << "commiting announces the change of: " << md->id();
+      // We announce the change we did, so all other parts of KMix can pick up the change
+      ControlManager::instance().announce(md->mixer()->id(), ControlChangeType::Volume, QString("Mixer.commitVolumeChange()"));
 }
 
 // @dbus, used also in kmix app
@@ -587,6 +643,7 @@ void Mixer::increaseOrDecreaseVolume( const QString& mixdeviceID, bool decrease
 
         _mixerBackend->writeVolumeToHW(mixdeviceID, md);
     }
+   ControlManager::instance().announce(md->mixer()->id(), ControlChangeType::Volume, QString("Mixer.increaseOrDecreaseVolume()"));
 
     /************************************************************
         It is important, not to implement this method like this:
@@ -611,7 +668,9 @@ bool Mixer::isDynamic()
 bool Mixer::moveStream( const QString id, const QString& destId )
 {
     // We should really check that id is within our md's....
-    return _mixerBackend->moveStream( id, destId );
+    bool ret = _mixerBackend->moveStream( id, destId );
+    ControlManager::instance().announce(QString(), ControlChangeType::ControlList, QString("Mixer.moveStream()"));
+    return ret;
 }
 
 #include "mixer.moc"
