@@ -25,42 +25,32 @@
 
 #include <kaction.h>
 #include <klocale.h>
-#include <kapplication.h>
 #include <kmenu.h>
-#include <kurl.h>
-#include <kglobalsettings.h>
-#include <kdialog.h>
-#include <kiconloader.h>
 #include <kdebug.h>
 #include <kwindowsystem.h>
 #include <kactioncollection.h>
 #include <ktoggleaction.h>
-#include <qapplication.h>
-#include <qcursor.h>
+
+#include <QDBusConnection>
+#include <QDBusConnectionInterface>
 #include <QDesktopWidget>
-#include <QMouseEvent>
+#include <QApplication>
 
-#ifdef Q_WS_X11
-#include <fixx11h.h>
-#endif
-
-#include <Phonon/MediaObject>
-
-#include "gui/dialogselectmaster.h"
 #include "apps/kmix.h"
+#include "core/ControlManager.h"
 #include "core/mixer.h"
-#include "gui/mixdevicewidget.h"
 #include "core/mixertoolbox.h"
+#include "gui/dialogselectmaster.h"
+#include "gui/mixdevicewidget.h"
 #include "gui/viewdockareapopup.h"
 
+
+//#define FEATURE_UNITY_POPUP true
+
 KMixDockWidget::KMixDockWidget(KMixWindow* parent, bool volumePopup)
     : KStatusNotifierItem(parent)
-  //  : KStatusNotifierItem(0)
-    , _audioPlayer(0L)
-    , _playBeepOnVolumeChange(false) // disabled due to triggering a "Bug"
     , _oldToolTipValue(-1)
     , _oldPixmapType('-')
-    , _volumePopup(volumePopup)
     , _kmixMainWindow(parent)
     , _contextMenuWasOpen(false)
 {
@@ -68,191 +58,173 @@ KMixDockWidget::KMixDockWidget(KMixWindow* parent, bool volumePopup)
     setTitle(i18n( "Volume Control"));
     setCategory(Hardware);
     setStatus(Active);
-    m_mixer = Mixer::getGlobalMasterMixer();  // ugly, but we'll live with that for now
-    createMasterVolWidget();
-    createActions();
+
+    // TODO Unity / Gnome only support one type of activation (left-click == right-click)
+    //      So we should show here the ViewDockAreaPopup instead of the menu:
+    //bool onlyOneMouseButtonAction = onlyHaveOneMouseButtonAction();
+
+    createMenuActions();
+
     connect(this, SIGNAL(scrollRequested(int,Qt::Orientation)), this, SLOT(trayWheelEvent(int,Qt::Orientation)));
     connect(this, SIGNAL(secondaryActivateRequested(QPoint)), this, SLOT(dockMute()));
 
-    bool NO_MENU_ANYMORE = true; 
-    
-    if ( NO_MENU_ANYMORE )
-    {
-      connect(contextMenu(), SIGNAL(aboutToShow()), this, SLOT(contextMenuAboutToShow()));
-    }
-    else
-    {
-      connect(this, SIGNAL(activateRequested(bool,QPoint)), this, SLOT(activateMenuOrWindow(bool,QPoint)));
-      connect(contextMenu(), SIGNAL(aboutToShow()), this, SLOT(contextMenuAboutToShow()));
-    }
+	kDebug() << "Construct the ViewDockAreaPopup and actions";
 
-#ifdef __GNUC__
-#warning minimizeRestore usage is currently slightly broken in KMIx. This should be fixed before doing a release.
-#endif
-    // TODO minimizeRestore usage is currently a bit broken. It only works by chance
+    bool onlyOneMouseButtonAction = !volumePopup;
+	if ( onlyOneMouseButtonAction)
+	{
+		// If only 1 mouse button is available, we will use it for the volume popup, and leave out the menu
+//		setContextMenu(0);
+		setAssociatedWidget(parent);
+		//setAssociatedWidget(new QWidget());
+		kDebug() << "We are now associated to " << associatedWidget();
+		_volWA = 0;
+		_referenceWidget = 0;
+	}
+	else
+	{
+		_referenceWidget = new KMenu(parent);
+		QWidget* referenceWindow;
+		if ( onlyOneMouseButtonAction )
+			referenceWindow = parent;
+		else
+			referenceWindow = _referenceWidget;
+		_referenceWidget2 = new ViewDockAreaPopup(referenceWindow, "dockArea", 0, QString("no-guiprofile-yet-in-dock"), parent);
+		_volWA = new QWidgetAction(_referenceWidget);
+		_volWA->setDefaultWidget(_referenceWidget2);
+		_referenceWidget->addAction(_volWA);
+		connect(contextMenu(), SIGNAL(aboutToShow()), this, SLOT(contextMenuAboutToShow()));
+	}
 
-    if (_volumePopup) {
-        kDebug() << "Construct the ViewDockAreaPopup and actions";
-        _referenceWidget = new KMenu(parent);
-        ViewDockAreaPopup* _referenceWidget2 = new ViewDockAreaPopup(_referenceWidget, "dockArea", Mixer::getGlobalMasterMixer(), 0, (GUIProfile*)0, parent);
-        _referenceWidget2->createDeviceWidgets();
-        connect(_referenceWidget2, SIGNAL(recreateMe()), _kmixMainWindow, SLOT(recreateDockWidget()));
-
-        _volWA = new QWidgetAction(_referenceWidget);
-        _volWA->setDefaultWidget(_referenceWidget2);
-        _referenceWidget->addAction(_volWA);
-
-        connect( m_mixer, SIGNAL(controlChanged()), _referenceWidget2, SLOT(refreshVolumeLevels()) );
-        //setAssociatedWidget(_referenceWidget);
-        //setAssociatedWidget(_referenceWidget);  // If you use the popup, associate that instead of the MainWindow
+  	ControlManager::instance().addListener(
+	  QString(), // All mixers (as the Global master Mixer might change)
+	(ControlChangeType::Type)(ControlChangeType::Volume | ControlChangeType::MasterChanged),
+	this,
+	QString("KMixDockWidget")	  
+	);
 	
-	//setContextMenu(_referenceWidget2);
-    }
-    else {
-        _volWA = 0;
-        _referenceWidget = 0;
-    }
+//	 ControlManager::instance().addListener(
+//	  QString(), // All mixers (as the Global master Mixer might change)
+//	ControlChangeType::MasterChanged,
+//	this,
+//	QString("KMixDockWidget")
+//	);
+	 
+	      // Refresh in all cases. When there is no Golbal Master we still need
+     // to initialize correctly (e.g. for showin 0% or hiding it)
+     refreshVolumeLevels();
 }
 
-
 KMixDockWidget::~KMixDockWidget()
 {
-    delete _audioPlayer;
+  ControlManager::instance().removeListener(this);
     // Note: deleting _volWA also deletes its associated ViewDockAreaPopup (_referenceWidget) and prevents the
     //       action to be left with a dangling pointer.
     //       cesken: I adapted the patch from https://bugs.kde.org/show_bug.cgi?id=220621#c27 to branch /branches/work/kmix 
     delete _volWA;
-
 }
 
-void KMixDockWidget::createActions()
+void KMixDockWidget::controlsChange(int changeType)
 {
-   QMenu *menu = contextMenu();
-   
-   shared_ptr<MixDevice> md = Mixer::getGlobalMasterMD();
-  if ( md.get() != 0 && md->playbackVolume().hasSwitch() )
+  ControlChangeType::Type type = ControlChangeType::fromInt(changeType);
+  switch (type )
   {
-    // Put "Mute" selector in context menu
-    KToggleAction *action = actionCollection()->add<KToggleAction>( "dock_mute" );
-    action->setText( i18n( "M&ute" ) );
-    connect(action, SIGNAL(triggered(bool)), SLOT(dockMute()));
-    menu->addAction( action );
-}
+    case  ControlChangeType::MasterChanged:
+      // Notify the main window, as it might need to update the visibiliy of the dock icon.
+//      _kmixMainWindow->updateDocking();
+//      _kmixMainWindow->saveConfig();
+      refreshVolumeLevels();
+      actionCollection()->action(QLatin1String("select_master"))->setEnabled(Mixer::getGlobalMasterMixer() != 0);
+      break;
 
-  // Put "Select Master Channel" dialog in context menu
-  if ( m_mixer != 0 ) {
-  QAction *action = actionCollection()->addAction( "select_master" );
-  action->setText( i18n("Select Master Channel...") );
-  connect(action, SIGNAL(triggered(bool)), SLOT(selectMaster()));
-  menu->addAction( action );
-  }
-  //Context menu entry to access phonon settings
-  menu->addAction(_kmixMainWindow->actionCollection()->action("launch_kdesoundsetup"));
+    case ControlChangeType::Volume:
+      refreshVolumeLevels();
+      break;
 
-   // Setup volume preview
-  if ( _playBeepOnVolumeChange ) {
-    _audioPlayer = Phonon::createPlayer(Phonon::MusicCategory);
-    _audioPlayer->setParent(this);
+    default:
+      ControlManager::warnUnexpectedChangeType(type, this);
   }
 }
 
-
-void
-KMixDockWidget::createMasterVolWidget()
+void KMixDockWidget::refreshVolumeLevels()
 {
-     // Reset flags, so that the dock icon will be reconstructed
-     _oldToolTipValue = -1;
-     _oldPixmapType   = '-';
+  setVolumeTip();
+  updatePixmap();
+}
 
-    if (Mixer::getGlobalMasterMD() == 0) {
-        // In case that there is no mixer installed, there will be no controlChanged() signal's
-        // Thus we prepare the dock areas manually
-        setVolumeTip();
-        updatePixmap();
-        return;
+void KMixDockWidget::createMenuActions()
+{
+    QMenu *menu = contextMenu();
+    if ( menu == 0)
+    	return; // We do not use a menu
+
+    shared_ptr<MixDevice> md = Mixer::getGlobalMasterMD();
+    if ( md.get() != 0 && md->hasMuteSwitch() ) {
+        // Put "Mute" selector in context menu
+        KToggleAction *action = actionCollection()->add<KToggleAction>( "dock_mute" );
+        updateDockMuteAction(action);
+        action->setText( i18n( "M&ute" ) );
+        connect(action, SIGNAL(triggered(bool)), SLOT(dockMute()));
+        menu->addAction( action );
     }
-    // create devices
 
-    m_mixer->readSetFromHWforceUpdate();  // after changing the master device, make sure to re-read (otherwise no "changed()" signals might get sent by the Mixer
-    /* With the recently introduced QSocketNotifier stuff, we can't rely on regular timer updates
-       any longer. Also the readSetFromHWforceUpdate() won't be enough. As a workaround, we trigger
-       all "repaints" manually here.
-       The call to m_mixer->readSetFromHWforceUpdate() is most likely superfluous, even if we don't use QSocketNotifier (e.g. in backends OSS, Solaris, ...)
-     */
-    setVolumeTip();
-    updatePixmap();
-    /* We are setting up 3 connections:
-     * Refreshig the _dockAreaPopup (not anymore necessary, because ViewBase already does it)
-     * Refreshing the Tooltip
-     * Refreshing the Icon
-     */
-    //    connect( m_mixer, SIGNAL(controlChanged()), _dockAreaPopup, SLOT(refreshVolumeLevels()) );
-    connect( m_mixer, SIGNAL(controlChanged()), this, SLOT(setVolumeTip()) );
-    connect( m_mixer, SIGNAL(controlChanged()), this, SLOT(updatePixmap()) );
+    // Put "Select Master Channel" dialog in context menu
+    QAction *action = actionCollection()->addAction( "select_master" );
+    action->setText( i18n("Select Master Channel...") );
+    action->setEnabled(Mixer::getGlobalMasterMixer() != 0);
+    connect(action, SIGNAL(triggered(bool)), SLOT(selectMaster()));
+    menu->addAction( action );
+
+    //Context menu entry to access phonon settings
+    menu->addAction(_kmixMainWindow->actionCollection()->action("launch_kdesoundsetup"));
 }
 
 void KMixDockWidget::selectMaster()
 {
-   DialogSelectMaster* dsm = new DialogSelectMaster(m_mixer);
+   DialogSelectMaster* dsm = new DialogSelectMaster(Mixer::getGlobalMasterMixer());
    dsm->setAttribute(Qt::WA_DeleteOnClose, true);
-   connect ( dsm, SIGNAL(newMasterSelected(QString&,QString&)), SLOT(handleNewMaster(QString&,QString&)) );
-   connect ( dsm, SIGNAL(newMasterSelected(QString&,QString&)), SIGNAL(newMasterSelected()) );
    dsm->show();
 }
 
 
-void KMixDockWidget::handleNewMaster(QString& /*mixerID*/, QString& /*control_id*/)
-{
-   /* When a new master is selected, we will need to destroy *this instance of KMixDockWidget.
-      Reason: This widget is a KSystemTrayIcon, and needs an associated QWidget. This is in
-      our case the ViewDockAreaPopup instance. As that is recreated, we need to recrate ourself as well.
-      The only chance to do so is outside, in fact it is done by KMixWindow::updateDocking(). This needs to
-      use deleteLater(), as we are executing in a SLOT currently.
-    */
-   _kmixMainWindow->updateDocking();
-}
-
-
 void
 KMixDockWidget::setVolumeTip()
 {
-	shared_ptr<MixDevice> md = Mixer::getGlobalMasterMD();
+    shared_ptr<MixDevice> md = Mixer::getGlobalMasterMD();
     QString tip = "";
-    int newToolTipValue = 0;
+    int virtualToolTipValue = 0;
 
     if ( md.get() == 0 )
     {
         tip = i18n("Mixer cannot be found"); // !! text could be reworked
-        newToolTipValue = -2;
+        virtualToolTipValue = -2;
     }
     else
     {
         // Playback volume will be used for the DockIcon if available.
         // This heuristic is "good enough" for the DockIcon for now.
-        Volume& vol = md->playbackVolume().hasVolume() ? md->playbackVolume() : md->captureVolume();
-       	int val = vol.getAvgVolumePercent(Volume::MALL);
+		int val = md->getUserfriendlyVolumeLevel();
+       	tip = i18n( "Volume at %1%", val );
+        if ( md->isMuted() )
+        	tip += i18n( " (Muted)" );
 
         // create a new "virtual" value. With that we see "volume changes" as well as "muted changes"
-        newToolTipValue = val;
-        if ( md->isMuted() ) newToolTipValue += 10000;
-        if ( _oldToolTipValue != newToolTipValue ) {
-            tip = i18n( "Volume at %1%", val );
-            if ( md->playbackVolume().hasSwitch() && md->isMuted() ) {
-                tip += i18n( " (Muted)" );
-            }
-        }
+        virtualToolTipValue = val;
+        if ( md->isMuted() )
+        	virtualToolTipValue += 10000;
     }
 
-    // The actual updating is only done when the "toolTipValue" was changed
-    if ( newToolTipValue != _oldToolTipValue ) {
+    // The actual updating is only done when the "toolTipValue" was changed (to avoid flicker)
+    if ( virtualToolTipValue != _oldToolTipValue )
+    {
         // changed (or completely new tooltip)
         setToolTipTitle(tip);
     }
-    _oldToolTipValue = newToolTipValue;
+    _oldToolTipValue = virtualToolTipValue;
 }
 
-void
-KMixDockWidget::updatePixmap()
+
+void KMixDockWidget::updatePixmap()
 {
 	shared_ptr<MixDevice> md = Mixer::getGlobalMasterMD();
 
@@ -263,20 +235,11 @@ KMixDockWidget::updatePixmap()
     }
     else
     {
-    	Volume& vol = md->playbackVolume().hasVolume() ? md->playbackVolume() : md->captureVolume();
-    	bool isInactive =  vol.isCapture() ? !md->isRecSource() : md->isMuted();
-		if ( isInactive )
-		{
-			newPixmapType = 'm';
-		}
-		else
-		{
-			int percentage         = vol.getAvgVolumePercent(Volume::MALL);
-			if      ( percentage <= 0 ) newPixmapType = '0';  // Hint: also negative-values
-			else if ( percentage < 25 ) newPixmapType = '1';
-			else if ( percentage < 75 ) newPixmapType = '2';
-			else                        newPixmapType = '3';
-	   }
+    	int percentage = md->getUserfriendlyVolumeLevel();
+		if      ( percentage <= 0 ) newPixmapType = '0';  // Hint: also negative-values
+		else if ( percentage < 25 ) newPixmapType = '1';
+		else if ( percentage < 75 ) newPixmapType = '2';
+		else                        newPixmapType = '3';
     }
 
    if ( newPixmapType != _oldPixmapType ) {
@@ -294,14 +257,6 @@ KMixDockWidget::updatePixmap()
    _oldPixmapType = newPixmapType;
 }
 
-
-
-void KMixDockWidget::activateMenuOrWindow(bool val, const QPoint &pos)
-{
-    kDebug() << "activateMenuOrWindow: " << val << "," << pos;
-}
-
-
 void KMixDockWidget::activate(const QPoint &pos)
 {
     kDebug() << "Activate at " << pos;
@@ -310,17 +265,21 @@ void KMixDockWidget::activate(const QPoint &pos)
     showHideMainWindow |= (_referenceWidget == 0);
     showHideMainWindow |= (pos.x() == 0  && pos.y() == 0);  // HACK. When the action comes from the context menu, the pos is (0,0)
 
-    if ( showHideMainWindow ) {
+    if ( showHideMainWindow )
+    {
         // Use default KStatusNotifierItem behavior if we are not using the dockAreaPopup
+    	// (or if the action comes from the context menu)
         kDebug() << "Use default KStatusNotifierItem behavior";
         setAssociatedWidget(_kmixMainWindow);
-//        KStatusNotifierItem::activate(pos);
-	KStatusNotifierItem::activate();
+        // This code path shows the Main Window (or hides it when it was shown)
+        KStatusNotifierItem::activate();
         return;
     }
 
+    // --- When this code path is executed, we want to show the DockAreaPopup)
+
     KMenu* dockAreaPopup =_referenceWidget; // TODO Refactor to use _referenceWidget directly
-    kDebug() << "Skip default KStatusNotifierItkdebem behavior";
+    kDebug() << "Skip default KStatusNotifierItem behavior";
     if ( dockAreaPopup->isVisible() ) {
         dockAreaPopup->hide();
         kDebug() << "dap is visible => hide and return";
@@ -351,9 +310,9 @@ void KMixDockWidget::activate(const QPoint &pos)
         }
 
         dockAreaPopup->move(x, y);  // so that the mouse is outside of the widget
-        kDebug() << "moving to" << dockAreaPopup->size() << x << y;
 
         dockAreaPopup->show();
+        kDebug() << "moving to" << dockAreaPopup->size() << x << y;
 
         // Now handle Multihead displays. And also make sure that the dialog is not
         // moved out-of-the screen on the right (see Bug 101742).
@@ -394,7 +353,6 @@ KMixDockWidget::trayWheelEvent(int delta,Qt::Orientation wheelOrientation)
     	delta = -delta;
 
     long int cv = inc * (delta / 120 );
-//    kDebug() << "twe: " << cv << " : " << vol;
     bool isInactive =  vol.isCapture() ? !md->isRecSource() : md->isMuted();
     kDebug() << "Operating on capture=" << vol.isCapture() << ", isInactive=" << isInactive;
 	if ( cv > 0 && isInactive)
@@ -408,66 +366,56 @@ KMixDockWidget::trayWheelEvent(int delta,Qt::Orientation wheelOrientation)
 	else
 	    vol.changeAllVolumes(cv);
 
-//	kDebug() << "twe: " << cv << " : " << vol;
-
-    if ( _playBeepOnVolumeChange ) {
-        QString fileName("KDE_Beep_Digital_1.ogg");
-        _audioPlayer->setCurrentSource(fileName);
-        _audioPlayer->play();
-    }
-
     md->mixer()->commitVolumeChange(md);
-    setVolumeTip();
+    refreshVolumeLevels();
 }
 
 
 void
 KMixDockWidget::dockMute()
 {
-	shared_ptr<MixDevice> md = Mixer::getGlobalMasterMD();
-	if ( md )
-   {
-      md->toggleMute();
-      md->mixer()->commitVolumeChange( md );
-   }
+    shared_ptr<MixDevice> md = Mixer::getGlobalMasterMD();
+    if ( md )
+    {
+        md->toggleMute();
+        md->mixer()->commitVolumeChange( md );
+        refreshVolumeLevels();
+    }
 }
 
-void
-KMixDockWidget::contextMenuAboutToShow()
+/**
+ * Returns whether the running Desktop only supports one Mouse Button
+ * Hint: Unity / Gnome only support one type of activation (left-click == right-click).
+ */
+bool KMixDockWidget::onlyHaveOneMouseButtonAction()
 {
-   // KStatusNotifierItem::contextMenuAboutToShow();
-    /*
-    kDebug() << "<<< mm 1";
-    QAction* showAction = actionCollection()->action("minimizeRestore");
-    kDebug() << "<<< mm 2";
-    if ( parent() && showAction )
-    {
-        if ( ((QWidget*)parent())->isVisible() ) // TODO isVisible() is not good enough here
-        {
-            showAction->setText( i18n("Hide Mixer Window") );
-        }
-        else
-        {
-            showAction->setText( i18n("Show Mixer Window") );
-        }
-        kDebug() << "<<< mm 3";
-        //disconnect(showAction, 0, 0, 0);
-        //connect(showAction, SIGNAL(triggered(bool),this,hideOrShowMainWindow());
-    }
-    */
+	QDBusConnection connection = QDBusConnection::sessionBus();
+    bool unityIsRunnig = (connection.interface()->isServiceRegistered("com.canonical.Unity.Panel.Service"));
+    // Possibly implement other detectors, like for Gnome 3 or Gnome 2
+    return unityIsRunnig;
 
+}
+
+void KMixDockWidget::contextMenuAboutToShow()
+{
     // Enable/Disable "Muted" menu item
-	shared_ptr<MixDevice> md = Mixer::getGlobalMasterMD();
+//	kDebug() << "hallo";
     KToggleAction *dockMuteAction = static_cast<KToggleAction*>(actionCollection()->action("dock_mute"));
-    //kDebug(67100) << "---> md=" << md << "dockMuteAction=" << dockMuteAction << "isMuted=" << md->isMuted();
-    if ( md != 0 && dockMuteAction != 0 ) {
-    	Volume& vol = md->playbackVolume().hasVolume() ? md->playbackVolume() : md->captureVolume();
-    	bool isInactive =  vol.isCapture() ? md->isMuted() : !md->isRecSource();
-        bool hasSwitch = vol.hasSwitch();
-        dockMuteAction->setEnabled( hasSwitch );
-        dockMuteAction->setChecked( hasSwitch && !isInactive );
-    }
+    updateDockMuteAction(dockMuteAction);
     _contextMenuWasOpen = true;
 }
 
+void KMixDockWidget::updateDockMuteAction ( KToggleAction* dockMuteAction )
+{  
+    shared_ptr<MixDevice> md = Mixer::getGlobalMasterMD();
+    if ( md != 0 && dockMuteAction != 0 )
+    {
+    	Volume& vol = md->playbackVolume().hasVolume() ? md->playbackVolume() : md->captureVolume();
+    	bool isInactive =  vol.isCapture() ? !md->isRecSource() : md->isMuted();
+        bool hasSwitch = vol.isCapture() ? vol.hasSwitch() : md->hasMuteSwitch();
+        dockMuteAction->setEnabled( hasSwitch );
+        dockMuteAction->setChecked( isInactive );
+    }
+}
+
 #include "kmixdockwidget.moc"
